export type HttpRequestInit = RequestInit & {
    config?: Record<string, unknown>;
    data?: object | null;
};
export type HttpHandler<T = unknown> = (req: RequestInfo, init?: HttpRequestInit) => T;
export interface Interceptor<TConfig, TRes, TOut> {
    init?: (config: TConfig) => void;
    preRequest?: (request: Request, config: TConfig) => Request;
    postRequest?: (response: TRes, config: TConfig) => TOut;
}
export type InterceptorApplier<TConfig, TIn, TOut> = (parent: HttpHandler<TIn>, config?: TConfig) => HttpHandler<TOut>;
export interface HttpClientWrapper<TIn> {
    wrap<TConfig, TOut>(interceptor: InterceptorApplier<TConfig, TIn, TOut>, config?: TConfig): HttpClient<TOut>;
}
export type HttpClient<T> = HttpHandler<T> & HttpClientWrapper<T>;
export declare function interceptor<TConfig = void, TIn = Promise<Response>, TOut = TIn>(handlers: {
    init?: (config?: TConfig) => void;
    preRequest?: (req: Request, config?: TConfig, init?: HttpRequestInit) => Request;
    postRequest?: (res: TIn, config?: TConfig) => TOut;
}): InterceptorApplier<TConfig, TIn, TOut>;
export declare function http<T = Promise<Response>>(parentClient?: HttpHandler<T>): HttpClient<T>;
export declare namespace http {
    var _a: () => HttpClient<Promise<object>>;
    export { _a as default };
}
export declare const httpJsonParser: InterceptorApplier<void, Promise<Response>, Promise<object>>;
export declare const httpBodySerialize: InterceptorApplier<void, Promise<Response>, Promise<Response>>;
export declare const httpErrorCode: InterceptorApplier<void, Promise<Response>, Promise<Response>>;
