import type { HttpHandler, HttpRequestInit } from './client';
type HttpMethod = 'GET' | 'POST' | 'PATCH' | 'PUT' | 'DELETE';
type HttpArgs = Record<string, string | number | boolean>;
export interface RestOptions {
    baseUrl: string;
    http: HttpHandler<Promise<object>>;
    parseArgs: (args: Record<string, string | number | boolean>) => string;
    substituteRootParams: (path: string, args: Record<string, string | number | boolean>) => string;
}
export type RestSchema<T> = ((res: object) => T) | T;
export interface RestMethodMemberDef<TArgs = Record<string, string | number | boolean>, TRes = unknown> {
    path: string;
    args?: TArgs;
    schema?: RestSchema<TRes>;
    config?: Record<string, unknown>;
}
type DefaultHttpResult = any;
export interface RestMemberDef<TArgs = Record<string, string | number | boolean>, TRes = DefaultHttpResult> extends RestMethodMemberDef<TArgs, TRes> {
    name: string;
    method: HttpMethod;
}
type InferRestRes<T> = T extends RestSchema<infer R> ? R : never;
export interface RestNamespaceDef {
    name: string;
    children: RestMember[];
}
type RestMember = RestMemberDef | RestNamespaceDef;
export type RestMethodFn<TArgs extends HttpArgs, TRes = unknown> = (args?: TArgs, config?: HttpRequestInit) => TRes;
type MapMember<M extends RestMember> = M extends RestNamespaceDef ? {
    [K in M['name']]: InferRest<M['children']>;
} : M extends RestMemberDef ? {
    [K in M['name']]: RestMethodFn<NonNullable<M['args']>, Promise<InferRestRes<NonNullable<M['schema']>>>>;
} : never;
type MapMembers<T extends readonly RestMember[]> = {
    [K in keyof T]: T[K] extends RestMember ? MapMember<T[K]> : never;
}[number];
type MergeUnion<U> = (U extends unknown ? (k: U) => void : never) extends (k: infer I) => void ? {
    [K in keyof I]: I[K];
} : never;
export type InferRest<T extends readonly RestMember[]> = MergeUnion<MapMembers<T>>;
declare const createFunctions: Readonly<{
    member: <TName extends string, X extends HttpMethod, TArgs = HttpArgs, TRes = object>(name: TName, method: X, opts?: RestMethodMemberDef<TArgs, TRes>) => {
        path: string;
        name: TName;
        method: X;
        args?: TArgs | undefined;
        schema?: RestSchema<TRes> | undefined;
        config?: Record<string, unknown>;
    };
    get: <T extends string, TArgs = HttpArgs, TRes = object>(name: T, config?: RestMethodMemberDef<TArgs, TRes> | undefined) => {
        path: string;
        name: T;
        method: "GET";
        args?: TArgs | undefined;
        schema?: RestSchema<TRes> | undefined;
        config?: Record<string, unknown>;
    };
    post: <T extends string, TArgs = HttpArgs, TRes = object>(name: T, config?: RestMethodMemberDef<TArgs, TRes> | undefined) => {
        path: string;
        name: T;
        method: "POST";
        args?: TArgs | undefined;
        schema?: RestSchema<TRes> | undefined;
        config?: Record<string, unknown>;
    };
    patch: <T extends string, TArgs = HttpArgs, TRes = object>(name: T, config?: RestMethodMemberDef<TArgs, TRes> | undefined) => {
        path: string;
        name: T;
        method: "PATCH";
        args?: TArgs | undefined;
        schema?: RestSchema<TRes> | undefined;
        config?: Record<string, unknown>;
    };
    put: <T extends string, TArgs = HttpArgs, TRes = object>(name: T, config?: RestMethodMemberDef<TArgs, TRes> | undefined) => {
        path: string;
        name: T;
        method: "PUT";
        args?: TArgs | undefined;
        schema?: RestSchema<TRes> | undefined;
        config?: Record<string, unknown>;
    };
    del: <T extends string, TArgs = HttpArgs, TRes = object>(name: T, config?: RestMethodMemberDef<TArgs, TRes> | undefined) => {
        path: string;
        name: T;
        method: "DELETE";
        args?: TArgs | undefined;
        schema?: RestSchema<TRes> | undefined;
        config?: Record<string, unknown>;
    };
    namespace: <TName extends string, TRest extends RestMember[]>(name: TName, createChildren: (fn: typeof createFunctions) => TRest) => {
        name: TName;
        children: TRest;
    };
}>;
export declare function rest(restOptions: Partial<RestOptions>): <T extends readonly RestMember[]>(path: string, define: (fn: typeof createFunctions) => T) => InferRest<T>;
export {};
